@@ -19,30 +19,22 @@
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-chdlc.c,v 1.43 2005-11-29 08:56:19 hannes Exp $ (LBL)";
-#endif
-
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <tcpdump-stdinc.h>
 
-#include <pcap.h>
-#include <stdio.h>
-
 #include "interface.h"
 #include "addrtoname.h"
 #include "ethertype.h"
 #include "extract.h"
-#include "ppp.h"
 #include "chdlc.h"
 
-static void chdlc_slarp_print(const u_char *, u_int);
+static void chdlc_slarp_print(netdissect_options *, const u_char *, u_int);
 
-static const struct tok chdlc_cast_values[] = { 
+static const struct tok chdlc_cast_values[] = {
     { CHDLC_UNICAST, "unicast" },
     { CHDLC_BCAST, "bcast" },
     { 0, NULL}
@@ -51,29 +43,29 @@ static const struct tok chdlc_cast_values[] = {
 
 /* Standard CHDLC printer */
 u_int
-chdlc_if_print(const struct pcap_pkthdr *h, register const u_char *p)
+chdlc_if_print(netdissect_options *ndo, const struct pcap_pkthdr *h, register const u_char *p)
 {
 	register u_int length = h->len;
 	register u_int caplen = h->caplen;
 
 	if (caplen < CHDLC_HDRLEN) {
-		printf("[|chdlc]");
+		ND_PRINT((ndo, "[|chdlc]"));
 		return (caplen);
 	}
-        return (chdlc_print(p,length));
+        return (chdlc_print(ndo, p,length));
 }
 
 u_int
-chdlc_print(register const u_char *p, u_int length) {
+chdlc_print(netdissect_options *ndo, register const u_char *p, u_int length) {
 	u_int proto;
 
 	proto = EXTRACT_16BITS(&p[2]);
-	if (eflag) {
-                printf("%s, ethertype %s (0x%04x), length %u: ",
+	if (ndo->ndo_eflag) {
+                ND_PRINT((ndo, "%s, ethertype %s (0x%04x), length %u: ",
                        tok2str(chdlc_cast_values, "0x%02x", p[0]),
                        tok2str(ethertype_values, "Unknown", proto),
                        proto,
-                       length);
+                       length));
 	}
 
 	length -= CHDLC_HDRLEN;
@@ -81,15 +73,15 @@ chdlc_print(register const u_char *p, u_int length) {
 
 	switch (proto) {
 	case ETHERTYPE_IP:
-		ip_print(gndo, p, length);
+		ip_print(ndo, p, length);
 		break;
 #ifdef INET6
 	case ETHERTYPE_IPV6:
-		ip6_print(gndo, p, length);
+		ip6_print(ndo, p, length);
 		break;
 #endif
 	case CHDLC_TYPE_SLARP:
-		chdlc_slarp_print(p, length);
+		chdlc_slarp_print(ndo, p, length);
 		break;
 #if 0
 	case CHDLC_TYPE_CDP:
@@ -98,20 +90,20 @@ chdlc_print(register const u_char *p, u_int length) {
 #endif
         case ETHERTYPE_MPLS:
         case ETHERTYPE_MPLS_MULTI:
-                mpls_print(p, length);
+                mpls_print(ndo, p, length);
 		break;
         case ETHERTYPE_ISO:
                 /* is the fudge byte set ? lets verify by spotting ISO headers */
                 if (*(p+1) == 0x81 ||
                     *(p+1) == 0x82 ||
                     *(p+1) == 0x83)
-                    isoclns_print(p+1, length-1, length-1);
+                    isoclns_print(ndo, p + 1, length - 1, length - 1);
                 else
-                    isoclns_print(p, length, length);
+                    isoclns_print(ndo, p, length, length);
                 break;
 	default:
-                if (!eflag)
-                        printf("unknown CHDLC protocol (0x%04x)", proto);
+                if (!ndo->ndo_eflag)
+                        ND_PRINT((ndo, "unknown CHDLC protocol (0x%04x)", proto));
                 break;
 	}
 
@@ -122,19 +114,19 @@ chdlc_print(register const u_char *p, u_int length) {
  * The fixed-length portion of a SLARP packet.
  */
 struct cisco_slarp {
-	u_int8_t code[4];
+	uint8_t code[4];
 #define SLARP_REQUEST	0
 #define SLARP_REPLY	1
 #define SLARP_KEEPALIVE	2
 	union {
 		struct {
-			u_int8_t addr[4];
-			u_int8_t mask[4];
+			uint8_t addr[4];
+			uint8_t mask[4];
 		} addr;
 		struct {
-			u_int8_t myseq[4];
-			u_int8_t yourseq[4];
-			u_int8_t rel[2];
+			uint8_t myseq[4];
+			uint8_t yourseq[4];
+			uint8_t rel[2];
 		} keep;
 	} un;
 };
@@ -143,20 +135,20 @@ struct cisco_slarp {
 #define SLARP_MAX_LEN	18
 
 static void
-chdlc_slarp_print(const u_char *cp, u_int length)
+chdlc_slarp_print(netdissect_options *ndo, const u_char *cp, u_int length)
 {
 	const struct cisco_slarp *slarp;
         u_int sec,min,hrs,days;
 
-        printf("SLARP (length: %u), ",length);
+	ND_PRINT((ndo, "SLARP (length: %u), ",length));
 	if (length < SLARP_MIN_LEN)
 		goto trunc;
 
 	slarp = (const struct cisco_slarp *)cp;
-	TCHECK2(*slarp, SLARP_MIN_LEN);
+	ND_TCHECK2(*slarp, SLARP_MIN_LEN);
 	switch (EXTRACT_32BITS(&slarp->code)) {
 	case SLARP_REQUEST:
-		printf("request");
+		ND_PRINT((ndo, "request"));
 		/*
 		 * At least according to William "Chops" Westfield's
 		 * message in
@@ -168,42 +160,41 @@ chdlc_slarp_print(const u_char *cp, u_int length)
 		 */
 		break;
 	case SLARP_REPLY:
-		printf("reply %s/%s",
-			ipaddr_string(&slarp->un.addr.addr),
-			ipaddr_string(&slarp->un.addr.mask));
+		ND_PRINT((ndo, "reply %s/%s",
+			ipaddr_string(ndo, &slarp->un.addr.addr),
+			ipaddr_string(ndo, &slarp->un.addr.mask)));
 		break;
 	case SLARP_KEEPALIVE:
-		printf("keepalive: mineseen=0x%08x, yourseen=0x%08x, reliability=0x%04x",
+		ND_PRINT((ndo, "keepalive: mineseen=0x%08x, yourseen=0x%08x, reliability=0x%04x",
                        EXTRACT_32BITS(&slarp->un.keep.myseq),
                        EXTRACT_32BITS(&slarp->un.keep.yourseq),
-                       EXTRACT_16BITS(&slarp->un.keep.rel));
+                       EXTRACT_16BITS(&slarp->un.keep.rel)));
 
                 if (length >= SLARP_MAX_LEN) { /* uptime-stamp is optional */
                         cp += SLARP_MIN_LEN;
-                        if (!TTEST2(*cp, 4))
-                                goto trunc;
+                        ND_TCHECK2(*cp, 4);
                         sec = EXTRACT_32BITS(cp) / 1000;
                         min = sec / 60; sec -= min * 60;
                         hrs = min / 60; min -= hrs * 60;
                         days = hrs / 24; hrs -= days * 24;
-                        printf(", link uptime=%ud%uh%um%us",days,hrs,min,sec);
+                        ND_PRINT((ndo, ", link uptime=%ud%uh%um%us",days,hrs,min,sec));
                 }
 		break;
 	default:
-		printf("0x%02x unknown", EXTRACT_32BITS(&slarp->code));
-                if (vflag <= 1)
-                    print_unknown_data(cp+4,"\n\t",length-4);
+		ND_PRINT((ndo, "0x%02x unknown", EXTRACT_32BITS(&slarp->code)));
+                if (ndo->ndo_vflag <= 1)
+                    print_unknown_data(ndo,cp+4,"\n\t",length-4);
 		break;
 	}
 
-	if (SLARP_MAX_LEN < length && vflag)
-		printf(", (trailing junk: %d bytes)", length - SLARP_MAX_LEN);
-        if (vflag > 1)
-            print_unknown_data(cp+4,"\n\t",length-4);
+	if (SLARP_MAX_LEN < length && ndo->ndo_vflag)
+		ND_PRINT((ndo, ", (trailing junk: %d bytes)", length - SLARP_MAX_LEN));
+        if (ndo->ndo_vflag > 1)
+            print_unknown_data(ndo,cp+4,"\n\t",length-4);
 	return;
 
 trunc:
-	printf("[|slarp]");
+	ND_PRINT((ndo, "[|slarp]"));
 }
 
 
