@@ -14,19 +14,13 @@
  * FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-eigrp.c,v 1.7 2005-05-06 02:53:26 guy Exp $";
-#endif
-
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <tcpdump-stdinc.h>
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 
 #include "interface.h"
@@ -39,13 +33,13 @@ static const char rcsid[] _U_ =
  */
 
 struct eigrp_common_header {
-    u_int8_t version;
-    u_int8_t opcode;
-    u_int8_t checksum[2];
-    u_int8_t flags[4];
-    u_int8_t seq[4];
-    u_int8_t ack[4];
-    u_int8_t asn[4];
+    uint8_t version;
+    uint8_t opcode;
+    uint8_t checksum[2];
+    uint8_t flags[4];
+    uint8_t seq[4];
+    uint8_t ack[4];
+    uint8_t asn[4];
 };
 
 #define	EIGRP_VERSION                        2
@@ -74,8 +68,8 @@ static const struct tok eigrp_common_header_flag_values[] = {
 };
 
 struct eigrp_tlv_header {
-    u_int8_t type[2];
-    u_int8_t length[2];
+    uint8_t type[2];
+    uint8_t length[2];
 };
 
 #define EIGRP_TLV_GENERAL_PARM   0x0001
@@ -108,91 +102,91 @@ static const struct tok eigrp_tlv_values[] = {
 };
 
 struct eigrp_tlv_general_parm_t {
-    u_int8_t k1;
-    u_int8_t k2;
-    u_int8_t k3;
-    u_int8_t k4;
-    u_int8_t k5;
-    u_int8_t res;
-    u_int8_t holdtime[2];
-};          
+    uint8_t k1;
+    uint8_t k2;
+    uint8_t k3;
+    uint8_t k4;
+    uint8_t k5;
+    uint8_t res;
+    uint8_t holdtime[2];
+};
 
 struct eigrp_tlv_sw_version_t {
-    u_int8_t ios_major;
-    u_int8_t ios_minor;
-    u_int8_t eigrp_major;
-    u_int8_t eigrp_minor;
-}; 
+    uint8_t ios_major;
+    uint8_t ios_minor;
+    uint8_t eigrp_major;
+    uint8_t eigrp_minor;
+};
 
 struct eigrp_tlv_ip_int_t {
-    u_int8_t nexthop[4];
-    u_int8_t delay[4];
-    u_int8_t bandwidth[4];
-    u_int8_t mtu[3];
-    u_int8_t hopcount;
-    u_int8_t reliability;
-    u_int8_t load;
-    u_int8_t reserved[2];
-    u_int8_t plen;
-    u_int8_t destination; /* variable length [1-4] bytes encoding */
-}; 
+    uint8_t nexthop[4];
+    uint8_t delay[4];
+    uint8_t bandwidth[4];
+    uint8_t mtu[3];
+    uint8_t hopcount;
+    uint8_t reliability;
+    uint8_t load;
+    uint8_t reserved[2];
+    uint8_t plen;
+    uint8_t destination; /* variable length [1-4] bytes encoding */
+};
 
 struct eigrp_tlv_ip_ext_t {
-    u_int8_t nexthop[4];
-    u_int8_t origin_router[4];
-    u_int8_t origin_as[4];
-    u_int8_t tag[4];
-    u_int8_t metric[4];
-    u_int8_t reserved[2];
-    u_int8_t proto_id;
-    u_int8_t flags;
-    u_int8_t delay[4];
-    u_int8_t bandwidth[4];
-    u_int8_t mtu[3];
-    u_int8_t hopcount;
-    u_int8_t reliability;
-    u_int8_t load;
-    u_int8_t reserved2[2];
-    u_int8_t plen;
-    u_int8_t destination; /* variable length [1-4] bytes encoding */
-}; 
+    uint8_t nexthop[4];
+    uint8_t origin_router[4];
+    uint8_t origin_as[4];
+    uint8_t tag[4];
+    uint8_t metric[4];
+    uint8_t reserved[2];
+    uint8_t proto_id;
+    uint8_t flags;
+    uint8_t delay[4];
+    uint8_t bandwidth[4];
+    uint8_t mtu[3];
+    uint8_t hopcount;
+    uint8_t reliability;
+    uint8_t load;
+    uint8_t reserved2[2];
+    uint8_t plen;
+    uint8_t destination; /* variable length [1-4] bytes encoding */
+};
 
 struct eigrp_tlv_at_cable_setup_t {
-    u_int8_t cable_start[2];
-    u_int8_t cable_end[2];
-    u_int8_t router_id[4];
+    uint8_t cable_start[2];
+    uint8_t cable_end[2];
+    uint8_t router_id[4];
 };
 
 struct eigrp_tlv_at_int_t {
-    u_int8_t nexthop[4];
-    u_int8_t delay[4];
-    u_int8_t bandwidth[4];
-    u_int8_t mtu[3];
-    u_int8_t hopcount;
-    u_int8_t reliability;
-    u_int8_t load;
-    u_int8_t reserved[2];
-    u_int8_t cable_start[2];
-    u_int8_t cable_end[2];
-}; 
+    uint8_t nexthop[4];
+    uint8_t delay[4];
+    uint8_t bandwidth[4];
+    uint8_t mtu[3];
+    uint8_t hopcount;
+    uint8_t reliability;
+    uint8_t load;
+    uint8_t reserved[2];
+    uint8_t cable_start[2];
+    uint8_t cable_end[2];
+};
 
 struct eigrp_tlv_at_ext_t {
-    u_int8_t nexthop[4];
-    u_int8_t origin_router[4];
-    u_int8_t origin_as[4];
-    u_int8_t tag[4];
-    u_int8_t proto_id;
-    u_int8_t flags;
-    u_int8_t metric[2];
-    u_int8_t delay[4];
-    u_int8_t bandwidth[4];
-    u_int8_t mtu[3];
-    u_int8_t hopcount;
-    u_int8_t reliability;
-    u_int8_t load;
-    u_int8_t reserved2[2];
-    u_int8_t cable_start[2];
-    u_int8_t cable_end[2];
+    uint8_t nexthop[4];
+    uint8_t origin_router[4];
+    uint8_t origin_as[4];
+    uint8_t tag[4];
+    uint8_t proto_id;
+    uint8_t flags;
+    uint8_t metric[2];
+    uint8_t delay[4];
+    uint8_t bandwidth[4];
+    uint8_t mtu[3];
+    uint8_t hopcount;
+    uint8_t reliability;
+    uint8_t load;
+    uint8_t reserved2[2];
+    uint8_t cable_start[2];
+    uint8_t cable_end[2];
 };
 
 static const struct tok eigrp_ext_proto_id_values[] = {
@@ -211,13 +205,13 @@ static const struct tok eigrp_ext_proto_id_values[] = {
 };
 
 void
-eigrp_print(register const u_char *pptr, register u_int len) {
+eigrp_print(netdissect_options *ndo, register const u_char *pptr, register u_int len) {
 
     const struct eigrp_common_header *eigrp_com_header;
     const struct eigrp_tlv_header *eigrp_tlv_header;
     const u_char *tptr,*tlv_tptr;
     u_int tlen,eigrp_tlv_len,eigrp_tlv_type,tlv_tlen, byte_length, bit_length;
-    u_int8_t prefix[4];
+    uint8_t prefix[4];
 
     union {
         const struct eigrp_tlv_general_parm_t *eigrp_tlv_general_parm;
@@ -231,21 +225,21 @@ eigrp_print(register const u_char *pptr, register u_int len) {
 
     tptr=pptr;
     eigrp_com_header = (const struct eigrp_common_header *)pptr;
-    TCHECK(*eigrp_com_header);
+    ND_TCHECK(*eigrp_com_header);
 
     /*
      * Sanity checking of the header.
      */
     if (eigrp_com_header->version != EIGRP_VERSION) {
-	printf("EIGRP version %u packet not supported",eigrp_com_header->version);
+	ND_PRINT((ndo, "EIGRP version %u packet not supported",eigrp_com_header->version));
 	return;
     }
 
     /* in non-verbose mode just lets print the basic Message Type*/
-    if (vflag < 1) {
-        printf("EIGRP %s, length: %u",
+    if (ndo->ndo_vflag < 1) {
+        ND_PRINT((ndo, "EIGRP %s, length: %u",
                tok2str(eigrp_opcode_values, "unknown (%u)",eigrp_com_header->opcode),
-               len);
+               len));
         return;
     }
 
@@ -254,7 +248,7 @@ eigrp_print(register const u_char *pptr, register u_int len) {
     tlen=len-sizeof(struct eigrp_common_header);
 
     /* FIXME print other header info */
-    printf("\n\tEIGRP v%u, opcode: %s (%u), chksum: 0x%04x, Flags: [%s]\n\tseq: 0x%08x, ack: 0x%08x, AS: %u, length: %u",
+    ND_PRINT((ndo, "\n\tEIGRP v%u, opcode: %s (%u), chksum: 0x%04x, Flags: [%s]\n\tseq: 0x%08x, ack: 0x%08x, AS: %u, length: %u",
            eigrp_com_header->version,
            tok2str(eigrp_opcode_values, "unknown, type: %u",eigrp_com_header->opcode),
            eigrp_com_header->opcode,
@@ -265,13 +259,13 @@ eigrp_print(register const u_char *pptr, register u_int len) {
            EXTRACT_32BITS(&eigrp_com_header->seq),
            EXTRACT_32BITS(&eigrp_com_header->ack),
            EXTRACT_32BITS(&eigrp_com_header->asn),
-           tlen);
+           tlen));
 
     tptr+=sizeof(const struct eigrp_common_header);
 
     while(tlen>0) {
         /* did we capture enough for fully decoding the object header ? */
-        TCHECK2(*tptr, sizeof(struct eigrp_tlv_header));
+        ND_TCHECK2(*tptr, sizeof(struct eigrp_tlv_header));
 
         eigrp_tlv_header = (const struct eigrp_tlv_header *)tptr;
         eigrp_tlv_len=EXTRACT_16BITS(&eigrp_tlv_header->length);
@@ -280,45 +274,45 @@ eigrp_print(register const u_char *pptr, register u_int len) {
 
         if (eigrp_tlv_len < sizeof(struct eigrp_tlv_header) ||
             eigrp_tlv_len > tlen) {
-            print_unknown_data(tptr+sizeof(struct eigrp_tlv_header),"\n\t    ",tlen);
+            print_unknown_data(ndo,tptr+sizeof(struct eigrp_tlv_header),"\n\t    ",tlen);
             return;
         }
 
-        printf("\n\t  %s TLV (0x%04x), length: %u",
+        ND_PRINT((ndo, "\n\t  %s TLV (0x%04x), length: %u",
                tok2str(eigrp_tlv_values,
                        "Unknown",
                        eigrp_tlv_type),
                eigrp_tlv_type,
-               eigrp_tlv_len);
+               eigrp_tlv_len));
 
         tlv_tptr=tptr+sizeof(struct eigrp_tlv_header);
         tlv_tlen=eigrp_tlv_len-sizeof(struct eigrp_tlv_header);
 
         /* did we capture enough for fully decoding the object ? */
-        TCHECK2(*tptr, eigrp_tlv_len);
+        ND_TCHECK2(*tptr, eigrp_tlv_len);
 
         switch(eigrp_tlv_type) {
 
         case EIGRP_TLV_GENERAL_PARM:
             tlv_ptr.eigrp_tlv_general_parm = (const struct eigrp_tlv_general_parm_t *)tlv_tptr;
 
-            printf("\n\t    holdtime: %us, k1 %u, k2 %u, k3 %u, k4 %u, k5 %u",
+            ND_PRINT((ndo, "\n\t    holdtime: %us, k1 %u, k2 %u, k3 %u, k4 %u, k5 %u",
                    EXTRACT_16BITS(tlv_ptr.eigrp_tlv_general_parm->holdtime),
                    tlv_ptr.eigrp_tlv_general_parm->k1,
                    tlv_ptr.eigrp_tlv_general_parm->k2,
                    tlv_ptr.eigrp_tlv_general_parm->k3,
                    tlv_ptr.eigrp_tlv_general_parm->k4,
-                   tlv_ptr.eigrp_tlv_general_parm->k5);
+                   tlv_ptr.eigrp_tlv_general_parm->k5));
             break;
 
         case EIGRP_TLV_SW_VERSION:
             tlv_ptr.eigrp_tlv_sw_version = (const struct eigrp_tlv_sw_version_t *)tlv_tptr;
 
-            printf("\n\t    IOS version: %u.%u, EIGRP version %u.%u",
+            ND_PRINT((ndo, "\n\t    IOS version: %u.%u, EIGRP version %u.%u",
                    tlv_ptr.eigrp_tlv_sw_version->ios_major,
                    tlv_ptr.eigrp_tlv_sw_version->ios_minor,
                    tlv_ptr.eigrp_tlv_sw_version->eigrp_major,
-                   tlv_ptr.eigrp_tlv_sw_version->eigrp_minor);
+                   tlv_ptr.eigrp_tlv_sw_version->eigrp_minor));
             break;
 
         case EIGRP_TLV_IP_INT:
@@ -326,28 +320,28 @@ eigrp_print(register const u_char *pptr, register u_int len) {
 
             bit_length = tlv_ptr.eigrp_tlv_ip_int->plen;
             if (bit_length > 32) {
-                printf("\n\t    illegal prefix length %u",bit_length);
+                ND_PRINT((ndo, "\n\t    illegal prefix length %u",bit_length));
                 break;
             }
             byte_length = (bit_length + 7) / 8; /* variable length encoding */
             memset(prefix, 0, 4);
             memcpy(prefix,&tlv_ptr.eigrp_tlv_ip_int->destination,byte_length);
 
-            printf("\n\t    IPv4 prefix: %15s/%u, nexthop: ",
-                   ipaddr_string(prefix),
-                   bit_length);
+            ND_PRINT((ndo, "\n\t    IPv4 prefix: %15s/%u, nexthop: ",
+                   ipaddr_string(ndo, prefix),
+                   bit_length));
             if (EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_ip_int->nexthop) == 0)
-                printf("self");
+                ND_PRINT((ndo, "self"));
             else
-                printf("%s",ipaddr_string(&tlv_ptr.eigrp_tlv_ip_int->nexthop));
+                ND_PRINT((ndo, "%s",ipaddr_string(ndo, &tlv_ptr.eigrp_tlv_ip_int->nexthop)));
 
-            printf("\n\t      delay %u ms, bandwidth %u Kbps, mtu %u, hop %u, reliability %u, load %u",
+            ND_PRINT((ndo, "\n\t      delay %u ms, bandwidth %u Kbps, mtu %u, hop %u, reliability %u, load %u",
                    (EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_ip_int->delay)/100),
                    EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_ip_int->bandwidth),
                    EXTRACT_24BITS(&tlv_ptr.eigrp_tlv_ip_int->mtu),
                    tlv_ptr.eigrp_tlv_ip_int->hopcount,
                    tlv_ptr.eigrp_tlv_ip_int->reliability,
-                   tlv_ptr.eigrp_tlv_ip_int->load);
+                   tlv_ptr.eigrp_tlv_ip_int->load));
             break;
 
         case EIGRP_TLV_IP_EXT:
@@ -355,99 +349,99 @@ eigrp_print(register const u_char *pptr, register u_int len) {
 
             bit_length = tlv_ptr.eigrp_tlv_ip_ext->plen;
             if (bit_length > 32) {
-                printf("\n\t    illegal prefix length %u",bit_length);
+                ND_PRINT((ndo, "\n\t    illegal prefix length %u",bit_length));
                 break;
             }
             byte_length = (bit_length + 7) / 8; /* variable length encoding */
             memset(prefix, 0, 4);
             memcpy(prefix,&tlv_ptr.eigrp_tlv_ip_ext->destination,byte_length);
 
-            printf("\n\t    IPv4 prefix: %15s/%u, nexthop: ",
-                   ipaddr_string(prefix),
-                   bit_length);
+            ND_PRINT((ndo, "\n\t    IPv4 prefix: %15s/%u, nexthop: ",
+                   ipaddr_string(ndo, prefix),
+                   bit_length));
             if (EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_ip_ext->nexthop) == 0)
-                printf("self");
+                ND_PRINT((ndo, "self"));
             else
-                printf("%s",ipaddr_string(&tlv_ptr.eigrp_tlv_ip_ext->nexthop));
+                ND_PRINT((ndo, "%s",ipaddr_string(ndo, &tlv_ptr.eigrp_tlv_ip_ext->nexthop)));
 
-            printf("\n\t      origin-router %s, origin-as %u, origin-proto %s, flags [0x%02x], tag 0x%08x, metric %u",
-                   ipaddr_string(tlv_ptr.eigrp_tlv_ip_ext->origin_router),
+            ND_PRINT((ndo, "\n\t      origin-router %s, origin-as %u, origin-proto %s, flags [0x%02x], tag 0x%08x, metric %u",
+                   ipaddr_string(ndo, tlv_ptr.eigrp_tlv_ip_ext->origin_router),
                    EXTRACT_32BITS(tlv_ptr.eigrp_tlv_ip_ext->origin_as),
                    tok2str(eigrp_ext_proto_id_values,"unknown",tlv_ptr.eigrp_tlv_ip_ext->proto_id),
                    tlv_ptr.eigrp_tlv_ip_ext->flags,
                    EXTRACT_32BITS(tlv_ptr.eigrp_tlv_ip_ext->tag),
-                   EXTRACT_32BITS(tlv_ptr.eigrp_tlv_ip_ext->metric));
+                   EXTRACT_32BITS(tlv_ptr.eigrp_tlv_ip_ext->metric)));
 
-            printf("\n\t      delay %u ms, bandwidth %u Kbps, mtu %u, hop %u, reliability %u, load %u",
+            ND_PRINT((ndo, "\n\t      delay %u ms, bandwidth %u Kbps, mtu %u, hop %u, reliability %u, load %u",
                    (EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_ip_ext->delay)/100),
                    EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_ip_ext->bandwidth),
                    EXTRACT_24BITS(&tlv_ptr.eigrp_tlv_ip_ext->mtu),
                    tlv_ptr.eigrp_tlv_ip_ext->hopcount,
                    tlv_ptr.eigrp_tlv_ip_ext->reliability,
-                   tlv_ptr.eigrp_tlv_ip_ext->load);
+                   tlv_ptr.eigrp_tlv_ip_ext->load));
             break;
 
         case EIGRP_TLV_AT_CABLE_SETUP:
             tlv_ptr.eigrp_tlv_at_cable_setup = (const struct eigrp_tlv_at_cable_setup_t *)tlv_tptr;
 
-            printf("\n\t    Cable-range: %u-%u, Router-ID %u",
+            ND_PRINT((ndo, "\n\t    Cable-range: %u-%u, Router-ID %u",
                    EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_cable_setup->cable_start),
                    EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_cable_setup->cable_end),
-                   EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_at_cable_setup->router_id));
+                   EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_at_cable_setup->router_id)));
             break;
 
         case EIGRP_TLV_AT_INT:
             tlv_ptr.eigrp_tlv_at_int = (const struct eigrp_tlv_at_int_t *)tlv_tptr;
 
-            printf("\n\t     Cable-Range: %u-%u, nexthop: ",
+            ND_PRINT((ndo, "\n\t     Cable-Range: %u-%u, nexthop: ",
                    EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_int->cable_start),
-                   EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_int->cable_end));
+                   EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_int->cable_end)));
 
             if (EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_at_int->nexthop) == 0)
-                printf("self");
+                ND_PRINT((ndo, "self"));
             else
-                printf("%u.%u",
+                ND_PRINT((ndo, "%u.%u",
                        EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_int->nexthop),
-                       EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_int->nexthop[2]));
+                       EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_int->nexthop[2])));
 
-            printf("\n\t      delay %u ms, bandwidth %u Kbps, mtu %u, hop %u, reliability %u, load %u",
+            ND_PRINT((ndo, "\n\t      delay %u ms, bandwidth %u Kbps, mtu %u, hop %u, reliability %u, load %u",
                    (EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_at_int->delay)/100),
                    EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_at_int->bandwidth),
                    EXTRACT_24BITS(&tlv_ptr.eigrp_tlv_at_int->mtu),
                    tlv_ptr.eigrp_tlv_at_int->hopcount,
                    tlv_ptr.eigrp_tlv_at_int->reliability,
-                   tlv_ptr.eigrp_tlv_at_int->load);
+                   tlv_ptr.eigrp_tlv_at_int->load));
             break;
 
         case EIGRP_TLV_AT_EXT:
             tlv_ptr.eigrp_tlv_at_ext = (const struct eigrp_tlv_at_ext_t *)tlv_tptr;
 
-            printf("\n\t     Cable-Range: %u-%u, nexthop: ",
+            ND_PRINT((ndo, "\n\t     Cable-Range: %u-%u, nexthop: ",
                    EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_ext->cable_start),
-                   EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_ext->cable_end));
+                   EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_ext->cable_end)));
 
             if (EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_at_ext->nexthop) == 0)
-                printf("self");
+                ND_PRINT((ndo, "self"));
             else
-                printf("%u.%u",
+                ND_PRINT((ndo, "%u.%u",
                        EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_ext->nexthop),
-                       EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_ext->nexthop[2]));
+                       EXTRACT_16BITS(&tlv_ptr.eigrp_tlv_at_ext->nexthop[2])));
 
-            printf("\n\t      origin-router %u, origin-as %u, origin-proto %s, flags [0x%02x], tag 0x%08x, metric %u",
+            ND_PRINT((ndo, "\n\t      origin-router %u, origin-as %u, origin-proto %s, flags [0x%02x], tag 0x%08x, metric %u",
                    EXTRACT_32BITS(tlv_ptr.eigrp_tlv_at_ext->origin_router),
                    EXTRACT_32BITS(tlv_ptr.eigrp_tlv_at_ext->origin_as),
                    tok2str(eigrp_ext_proto_id_values,"unknown",tlv_ptr.eigrp_tlv_at_ext->proto_id),
                    tlv_ptr.eigrp_tlv_at_ext->flags,
                    EXTRACT_32BITS(tlv_ptr.eigrp_tlv_at_ext->tag),
-                   EXTRACT_16BITS(tlv_ptr.eigrp_tlv_at_ext->metric));
+                   EXTRACT_16BITS(tlv_ptr.eigrp_tlv_at_ext->metric)));
 
-            printf("\n\t      delay %u ms, bandwidth %u Kbps, mtu %u, hop %u, reliability %u, load %u",
+            ND_PRINT((ndo, "\n\t      delay %u ms, bandwidth %u Kbps, mtu %u, hop %u, reliability %u, load %u",
                    (EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_at_ext->delay)/100),
                    EXTRACT_32BITS(&tlv_ptr.eigrp_tlv_at_ext->bandwidth),
                    EXTRACT_24BITS(&tlv_ptr.eigrp_tlv_at_ext->mtu),
                    tlv_ptr.eigrp_tlv_at_ext->hopcount,
                    tlv_ptr.eigrp_tlv_at_ext->reliability,
-                   tlv_ptr.eigrp_tlv_at_ext->load);
+                   tlv_ptr.eigrp_tlv_at_ext->load));
             break;
 
             /*
@@ -462,13 +456,13 @@ eigrp_print(register const u_char *pptr, register u_int len) {
         case EIGRP_TLV_IPX_EXT:
 
         default:
-            if (vflag <= 1)
-                print_unknown_data(tlv_tptr,"\n\t    ",tlv_tlen);
+            if (ndo->ndo_vflag <= 1)
+                print_unknown_data(ndo,tlv_tptr,"\n\t    ",tlv_tlen);
             break;
         }
         /* do we want to see an additionally hexdump ? */
-        if (vflag > 1)
-            print_unknown_data(tptr+sizeof(struct eigrp_tlv_header),"\n\t    ",
+        if (ndo->ndo_vflag > 1)
+            print_unknown_data(ndo,tptr+sizeof(struct eigrp_tlv_header),"\n\t    ",
                                eigrp_tlv_len-sizeof(struct eigrp_tlv_header));
 
         tptr+=eigrp_tlv_len;
@@ -476,5 +470,5 @@ eigrp_print(register const u_char *pptr, register u_int len) {
     }
     return;
 trunc:
-    printf("\n\t\t packet exceeded snapshot");
+    ND_PRINT((ndo, "\n\t\t packet exceeded snapshot"));
 }
