@@ -19,28 +19,24 @@
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-ip.c,v 1.159 2007-09-14 01:29:28 guy Exp $ (LBL)";
-#endif
-
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <tcpdump-stdinc.h>
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 
-#include "addrtoname.h"
 #include "interface.h"
+#include "addrtoname.h"
 #include "extract.h"			/* must come after interface.h */
 
 #include "ip.h"
 #include "ipproto.h"
 
+static const char tstr[] = "[|ip]";
+
 static const struct tok ip_option_values[] = {
     { IPOPT_EOL, "EOL" },
     { IPOPT_NOP, "NOP" },
@@ -58,25 +54,26 @@ static const struct tok ip_option_values[] = {
  * print the recorded route in an IP RR, LSRR or SSRR option.
  */
 static void
-ip_printroute(register const u_char *cp, u_int length)
+ip_printroute(netdissect_options *ndo,
+              register const u_char *cp, u_int length)
 {
 	register u_int ptr;
 	register u_int len;
 
 	if (length < 3) {
-		printf(" [bad length %u]", length);
+		ND_PRINT((ndo, " [bad length %u]", length));
 		return;
 	}
 	if ((length + 1) & 3)
-		printf(" [bad length %u]", length);
+		ND_PRINT((ndo, " [bad length %u]", length));
 	ptr = cp[2] - 1;
 	if (ptr < 3 || ((ptr + 1) & 3) || ptr > length + 1)
-		printf(" [bad ptr %u]", cp[2]);
+		ND_PRINT((ndo, " [bad ptr %u]", cp[2]));
 
 	for (len = 3; len < length; len += 4) {
-		printf(" %s", ipaddr_string(&cp[len]));
-                if (ptr > len)
-                        printf(",");
+		ND_PRINT((ndo, " %s", ipaddr_string(ndo, &cp[len])));
+		if (ptr > len)
+			ND_PRINT((ndo, ","));
 	}
 }
 
@@ -87,13 +84,14 @@ ip_printroute(register const u_char *cp, u_int length)
  * This is used for UDP and TCP pseudo-header in the checksum
  * calculation.
  */
-static u_int32_t
-ip_finddst(const struct ip *ip)
+static uint32_t
+ip_finddst(netdissect_options *ndo,
+           const struct ip *ip)
 {
 	int length;
 	int len;
 	const u_char *cp;
-	u_int32_t retval;
+	uint32_t retval;
 
 	cp = (const u_char *)(ip + 1);
 	length = (IP_HL(ip) << 2) - sizeof(struct ip);
@@ -101,31 +99,31 @@ ip_finddst(const struct ip *ip)
 	for (; length > 0; cp += len, length -= len) {
 		int tt;
 
-		TCHECK(*cp);
+		ND_TCHECK(*cp);
 		tt = *cp;
 		if (tt == IPOPT_EOL)
 			break;
 		else if (tt == IPOPT_NOP)
 			len = 1;
 		else {
-			TCHECK(cp[1]);
+			ND_TCHECK(cp[1]);
 			len = cp[1];
 			if (len < 2)
 				break;
 		}
-		TCHECK2(*cp, len);
+		ND_TCHECK2(*cp, len);
 		switch (tt) {
 
 		case IPOPT_SSRR:
 		case IPOPT_LSRR:
 			if (len < 7)
 				break;
-			memcpy(&retval, cp + len - 4, 4);
+			UNALIGNED_MEMCPY(&retval, cp + len - 4, 4);
 			return retval;
 		}
 	}
 trunc:
-	memcpy(&retval, &ip->ip_dst.s_addr, sizeof(u_int32_t));
+	UNALIGNED_MEMCPY(&retval, &ip->ip_dst.s_addr, sizeof(uint32_t));
 	return retval;
 }
 
@@ -133,37 +131,39 @@ trunc:
  * Compute a V4-style checksum by building a pseudoheader.
  */
 int
-nextproto4_cksum(const struct ip *ip, const u_int8_t *data,
-		 u_int len, u_int next_proto)
+nextproto4_cksum(netdissect_options *ndo,
+                 const struct ip *ip, const uint8_t *data,
+                 u_int len, u_int covlen, u_int next_proto)
 {
 	struct phdr {
-		u_int32_t src;
-		u_int32_t dst;
+		uint32_t src;
+		uint32_t dst;
 		u_char mbz;
 		u_char proto;
-		u_int16_t len;
+		uint16_t len;
 	} ph;
 	struct cksum_vec vec[2];
 
 	/* pseudo-header.. */
-	ph.len = htons((u_int16_t)len);
+	ph.len = htons((uint16_t)len);
 	ph.mbz = 0;
 	ph.proto = next_proto;
-	memcpy(&ph.src, &ip->ip_src.s_addr, sizeof(u_int32_t));
+	UNALIGNED_MEMCPY(&ph.src, &ip->ip_src.s_addr, sizeof(uint32_t));
 	if (IP_HL(ip) == 5)
-		memcpy(&ph.dst, &ip->ip_dst.s_addr, sizeof(u_int32_t));
+		UNALIGNED_MEMCPY(&ph.dst, &ip->ip_dst.s_addr, sizeof(uint32_t));
 	else
-		ph.dst = ip_finddst(ip);
+		ph.dst = ip_finddst(ndo, ip);
 
-	vec[0].ptr = (const u_int8_t *)(void *)&ph;
+	vec[0].ptr = (const uint8_t *)(void *)&ph;
 	vec[0].len = sizeof(ph);
 	vec[1].ptr = data;
-	vec[1].len = len;
+	vec[1].len = covlen;
 	return (in_cksum(vec, 2));
 }
 
 static void
-ip_printts(register const u_char *cp, u_int length)
+ip_printts(netdissect_options *ndo,
+           register const u_char *cp, u_int length)
 {
 	register u_int ptr;
 	register u_int len;
@@ -171,23 +171,23 @@ ip_printts(register const u_char *cp, u_int length)
 	const char *type;
 
 	if (length < 4) {
-		printf("[bad length %u]", length);
+		ND_PRINT((ndo, "[bad length %u]", length));
 		return;
 	}
-	printf(" TS{");
+	ND_PRINT((ndo, " TS{"));
 	hoplen = ((cp[3]&0xF) != IPOPT_TS_TSONLY) ? 8 : 4;
 	if ((length - 4) & (hoplen-1))
-		printf("[bad length %u]", length);
+		ND_PRINT((ndo, "[bad length %u]", length));
 	ptr = cp[2] - 1;
 	len = 0;
 	if (ptr < 4 || ((ptr - 4) & (hoplen-1)) || ptr > length + 1)
-		printf("[bad ptr %u]", cp[2]);
+		ND_PRINT((ndo, "[bad ptr %u]", cp[2]));
 	switch (cp[3]&0xF) {
 	case IPOPT_TS_TSONLY:
-		printf("TSONLY");
+		ND_PRINT((ndo, "TSONLY"));
 		break;
 	case IPOPT_TS_TSANDADDR:
-		printf("TS+ADDR");
+		ND_PRINT((ndo, "TS+ADDR"));
 		break;
 	/*
 	 * prespecified should really be 3, but some ones might send 2
@@ -196,13 +196,13 @@ ip_printts(register const u_char *cp, u_int length)
 	 */
 
 	case 2:
-		printf("PRESPEC2.0");
+		ND_PRINT((ndo, "PRESPEC2.0"));
 		break;
 	case 3:			/* IPOPT_TS_PRESPEC */
-		printf("PRESPEC");
+		ND_PRINT((ndo, "PRESPEC"));
 		break;
 	default:
-		printf("[bad ts type %d]", cp[3]&0xF);
+		ND_PRINT((ndo, "[bad ts type %d]", cp[3]&0xF));
 		goto done;
 	}
 
@@ -210,25 +210,26 @@ ip_printts(register const u_char *cp, u_int length)
 	for (len = 4; len < length; len += hoplen) {
 		if (ptr == len)
 			type = " ^ ";
-		printf("%s%d@%s", type, EXTRACT_32BITS(&cp[len+hoplen-4]),
-		       hoplen!=8 ? "" : ipaddr_string(&cp[len]));
+		ND_PRINT((ndo, "%s%d@%s", type, EXTRACT_32BITS(&cp[len+hoplen-4]),
+		       hoplen!=8 ? "" : ipaddr_string(ndo, &cp[len])));
 		type = " ";
 	}
 
 done:
-	printf("%s", ptr == len ? " ^ " : "");
+	ND_PRINT((ndo, "%s", ptr == len ? " ^ " : ""));
 
 	if (cp[3]>>4)
-		printf(" [%d hops not recorded]} ", cp[3]>>4);
+		ND_PRINT((ndo, " [%d hops not recorded]} ", cp[3]>>4));
 	else
-		printf("}");
+		ND_PRINT((ndo, "}"));
 }
 
 /*
  * print IP options.
  */
 static void
-ip_optprint(register const u_char *cp, u_int length)
+ip_optprint(netdissect_options *ndo,
+            register const u_char *cp, u_int length)
 {
 	register u_int option_len;
 	const char *sep = "";
@@ -236,57 +237,57 @@ ip_optprint(register const u_char *cp, u_int length)
 	for (; length > 0; cp += option_len, length -= option_len) {
 		u_int option_code;
 
-		printf("%s", sep);
+		ND_PRINT((ndo, "%s", sep));
 		sep = ",";
 
-		TCHECK(*cp);
+		ND_TCHECK(*cp);
 		option_code = *cp;
 
-                printf("%s",
-                        tok2str(ip_option_values,"unknown %u",option_code));
+		ND_PRINT((ndo, "%s",
+		          tok2str(ip_option_values,"unknown %u",option_code)));
 
 		if (option_code == IPOPT_NOP ||
                     option_code == IPOPT_EOL)
 			option_len = 1;
 
 		else {
-			TCHECK(cp[1]);
+			ND_TCHECK(cp[1]);
 			option_len = cp[1];
 			if (option_len < 2) {
-		                printf(" [bad length %u]", option_len);
+				ND_PRINT((ndo, " [bad length %u]", option_len));
 				return;
 			}
 		}
 
 		if (option_len > length) {
-	                printf(" [bad length %u]", option_len);
+			ND_PRINT((ndo, " [bad length %u]", option_len));
 			return;
 		}
 
-                TCHECK2(*cp, option_len);
+		ND_TCHECK2(*cp, option_len);
 
 		switch (option_code) {
 		case IPOPT_EOL:
 			return;
 
 		case IPOPT_TS:
-			ip_printts(cp, option_len);
+			ip_printts(ndo, cp, option_len);
 			break;
 
 		case IPOPT_RR:       /* fall through */
 		case IPOPT_SSRR:
 		case IPOPT_LSRR:
-			ip_printroute(cp, option_len);
+			ip_printroute(ndo, cp, option_len);
 			break;
 
 		case IPOPT_RA:
 			if (option_len < 4) {
-				printf(" [bad length %u]", option_len);
+				ND_PRINT((ndo, " [bad length %u]", option_len));
 				break;
 			}
-                        TCHECK(cp[3]);
-                        if (EXTRACT_16BITS(&cp[2]) != 0)
-                            printf(" value %u", EXTRACT_16BITS(&cp[2]));
+			ND_TCHECK(cp[3]);
+			if (EXTRACT_16BITS(&cp[2]) != 0)
+				ND_PRINT((ndo, " value %u", EXTRACT_16BITS(&cp[2])));
 			break;
 
 		case IPOPT_NOP:       /* nothing to print - fall through */
@@ -298,7 +299,7 @@ ip_optprint(register const u_char *cp, u_int length)
 	return;
 
 trunc:
-	printf("[|ip]");
+	ND_PRINT((ndo, "%s", tstr));
 }
 
 #define IP_RES 0x8000
@@ -330,7 +331,7 @@ again:
 
 	case IPPROTO_AH:
 		ipds->nh = *ipds->cp;
-		ipds->advance = ah_print(ipds->cp);
+		ipds->advance = ah_print(ndo, ipds->cp);
 		if (ipds->advance <= 0)
 			break;
 		ipds->cp += ipds->advance;
@@ -354,7 +355,7 @@ again:
 	case IPPROTO_IPCOMP:
 	{
 		int enh;
-		ipds->advance = ipcomp_print(ipds->cp, &enh);
+		ipds->advance = ipcomp_print(ndo, ipds->cp, &enh);
 		if (ipds->advance <= 0)
 			break;
 		ipds->cp += ipds->advance;
@@ -364,28 +365,28 @@ again:
 	}
 
 	case IPPROTO_SCTP:
-		sctp_print(ipds->cp, (const u_char *)ipds->ip, ipds->len);
+		sctp_print(ndo, ipds->cp, (const u_char *)ipds->ip, ipds->len);
 		break;
 
 	case IPPROTO_DCCP:
-		dccp_print(ipds->cp, (const u_char *)ipds->ip, ipds->len);
+		dccp_print(ndo, ipds->cp, (const u_char *)ipds->ip, ipds->len);
 		break;
 
 	case IPPROTO_TCP:
 		/* pass on the MF bit plus the offset to detect fragments */
-		tcp_print(ipds->cp, ipds->len, (const u_char *)ipds->ip,
+		tcp_print(ndo, ipds->cp, ipds->len, (const u_char *)ipds->ip,
 			  ipds->off & (IP_MF|IP_OFFMASK));
 		break;
 
 	case IPPROTO_UDP:
 		/* pass on the MF bit plus the offset to detect fragments */
-		udp_print(ipds->cp, ipds->len, (const u_char *)ipds->ip,
+		udp_print(ndo, ipds->cp, ipds->len, (const u_char *)ipds->ip,
 			  ipds->off & (IP_MF|IP_OFFMASK));
 		break;
 
 	case IPPROTO_ICMP:
 		/* pass on the MF bit plus the offset to detect fragments */
-		icmp_print(ipds->cp, ipds->len, (const u_char *)ipds->ip,
+		icmp_print(ndo, ipds->cp, ipds->len, (const u_char *)ipds->ip,
 			   ipds->off & (IP_MF|IP_OFFMASK));
 		break;
 
@@ -403,11 +404,11 @@ again:
 		 * match was the current protocol number
 		 * assignments say.
 		 */
-		igrp_print(ipds->cp, ipds->len, (const u_char *)ipds->ip);
+		igrp_print(ndo, ipds->cp, ipds->len);
 		break;
 
 	case IPPROTO_EIGRP:
-		eigrp_print(ipds->cp, ipds->len);
+		eigrp_print(ndo, ipds->cp, ipds->len);
 		break;
 
 	case IPPROTO_ND:
@@ -415,21 +416,21 @@ again:
 		break;
 
 	case IPPROTO_EGP:
-		egp_print(ipds->cp, ipds->len);
+		egp_print(ndo, ipds->cp, ipds->len);
 		break;
 
 	case IPPROTO_OSPF:
-		ospf_print(ipds->cp, ipds->len, (const u_char *)ipds->ip);
+		ospf_print(ndo, ipds->cp, ipds->len, (const u_char *)ipds->ip);
 		break;
 
 	case IPPROTO_IGMP:
-		igmp_print(ipds->cp, ipds->len);
+		igmp_print(ndo, ipds->cp, ipds->len);
 		break;
 
 	case IPPROTO_IPV4:
 		/* DVMRP multicast tunnel (ip-in-ip encapsulation) */
 		ip_print(ndo, ipds->cp, ipds->len);
-		if (! vflag) {
+		if (! ndo->ndo_vflag) {
 			ND_PRINT((ndo, " (ipip-proto-4)"));
 			return;
 		}
@@ -443,42 +444,43 @@ again:
 #endif /*INET6*/
 
 	case IPPROTO_RSVP:
-		rsvp_print(ipds->cp, ipds->len);
+		rsvp_print(ndo, ipds->cp, ipds->len);
 		break;
 
 	case IPPROTO_GRE:
 		/* do it */
-		gre_print(ipds->cp, ipds->len);
+		gre_print(ndo, ipds->cp, ipds->len);
 		break;
 
 	case IPPROTO_MOBILE:
-		mobile_print(ipds->cp, ipds->len);
+		mobile_print(ndo, ipds->cp, ipds->len);
 		break;
 
 	case IPPROTO_PIM:
 		vec[0].ptr = ipds->cp;
 		vec[0].len = ipds->len;
-		pim_print(ipds->cp, ipds->len, in_cksum(vec, 1));
+		pim_print(ndo, ipds->cp, ipds->len, in_cksum(vec, 1));
 		break;
 
 	case IPPROTO_VRRP:
-		if (packettype == PT_CARP) {
-			if (vflag)
-				(void)printf("carp %s > %s: ",
-					     ipaddr_string(&ipds->ip->ip_src),
-					     ipaddr_string(&ipds->ip->ip_dst));
-			carp_print(ipds->cp, ipds->len, ipds->ip->ip_ttl);
+		if (ndo->ndo_packettype == PT_CARP) {
+			if (ndo->ndo_vflag)
+				ND_PRINT((ndo, "carp %s > %s: ",
+					     ipaddr_string(ndo, &ipds->ip->ip_src),
+					     ipaddr_string(ndo, &ipds->ip->ip_dst)));
+			carp_print(ndo, ipds->cp, ipds->len, ipds->ip->ip_ttl);
 		} else {
-			if (vflag)
-				(void)printf("vrrp %s > %s: ",
-					     ipaddr_string(&ipds->ip->ip_src),
-					     ipaddr_string(&ipds->ip->ip_dst));
-			vrrp_print(ipds->cp, ipds->len, ipds->ip->ip_ttl);
+			if (ndo->ndo_vflag)
+				ND_PRINT((ndo, "vrrp %s > %s: ",
+					     ipaddr_string(ndo, &ipds->ip->ip_src),
+					     ipaddr_string(ndo, &ipds->ip->ip_dst)));
+			vrrp_print(ndo, ipds->cp, ipds->len,
+				(const u_char *)ipds->ip, ipds->ip->ip_ttl);
 		}
 		break;
 
 	case IPPROTO_PGM:
-		pgm_print(ipds->cp, ipds->len, (const u_char *)ipds->ip);
+		pgm_print(ndo, ipds->cp, ipds->len, (const u_char *)ipds->ip);
 		break;
 
 	default:
@@ -523,40 +525,40 @@ ip_print(netdissect_options *ndo,
 	const u_char *ipend;
 	u_int hlen;
 	struct cksum_vec vec[1];
-	u_int16_t sum, ip_sum;
+	uint16_t sum, ip_sum;
 	struct protoent *proto;
 
 	ipds->ip = (const struct ip *)bp;
 	if (IP_V(ipds->ip) != 4) { /* print version if != 4 */
-	    printf("IP%u ", IP_V(ipds->ip));
+	    ND_PRINT((ndo, "IP%u ", IP_V(ipds->ip)));
 	    if (IP_V(ipds->ip) == 6)
-		printf(", wrong link-layer encapsulation");
+	      ND_PRINT((ndo, ", wrong link-layer encapsulation"));
 	}
-        else if (!eflag)
-	    printf("IP ");
+	else if (!ndo->ndo_eflag)
+		ND_PRINT((ndo, "IP "));
 
 	if ((u_char *)(ipds->ip + 1) > ndo->ndo_snapend) {
-		printf("[|ip]");
+		ND_PRINT((ndo, "%s", tstr));
 		return;
 	}
 	if (length < sizeof (struct ip)) {
-		(void)printf("truncated-ip %u", length);
+		ND_PRINT((ndo, "truncated-ip %u", length));
 		return;
 	}
 	hlen = IP_HL(ipds->ip) * 4;
 	if (hlen < sizeof (struct ip)) {
-		(void)printf("bad-hlen %u", hlen);
+		ND_PRINT((ndo, "bad-hlen %u", hlen));
 		return;
 	}
 
 	ipds->len = EXTRACT_16BITS(&ipds->ip->ip_len);
 	if (length < ipds->len)
-		(void)printf("truncated-ip - %u bytes missing! ",
-			ipds->len - length);
+		ND_PRINT((ndo, "truncated-ip - %u bytes missing! ",
+			ipds->len - length));
 	if (ipds->len < hlen) {
 #ifdef GUESS_TSO
             if (ipds->len) {
-                (void)printf("bad-len %u", ipds->len);
+                ND_PRINT((ndo, "bad-len %u", ipds->len));
                 return;
             }
             else {
@@ -564,7 +566,7 @@ ip_print(netdissect_options *ndo,
                 ipds->len = length;
             }
 #else
-            (void)printf("bad-len %u", ipds->len);
+            ND_PRINT((ndo, "bad-len %u", ipds->len));
             return;
 #endif /* GUESS_TSO */
 	}
@@ -580,24 +582,24 @@ ip_print(netdissect_options *ndo,
 
 	ipds->off = EXTRACT_16BITS(&ipds->ip->ip_off);
 
-        if (vflag) {
-            (void)printf("(tos 0x%x", (int)ipds->ip->ip_tos);
+        if (ndo->ndo_vflag) {
+            ND_PRINT((ndo, "(tos 0x%x", (int)ipds->ip->ip_tos));
             /* ECN bits */
             if (ipds->ip->ip_tos & 0x03) {
                 switch (ipds->ip->ip_tos & 0x03) {
                 case 1:
-                    (void)printf(",ECT(1)");
+                    ND_PRINT((ndo, ",ECT(1)"));
                     break;
                 case 2:
-                    (void)printf(",ECT(0)");
+                    ND_PRINT((ndo, ",ECT(0)"));
                     break;
                 case 3:
-                    (void)printf(",CE");
+                    ND_PRINT((ndo, ",CE"));
                 }
             }
 
             if (ipds->ip->ip_ttl >= 1)
-                (void)printf(", ttl %u", ipds->ip->ip_ttl);
+                ND_PRINT((ndo, ", ttl %u", ipds->ip->ip_ttl));
 
 	    /*
 	     * for the firewall guys, print id, offset.
@@ -605,33 +607,33 @@ ip_print(netdissect_options *ndo,
 	     * For unfragmented datagrams, note the don't fragment flag.
 	     */
 
-	    (void)printf(", id %u, offset %u, flags [%s], proto %s (%u)",
+	    ND_PRINT((ndo, ", id %u, offset %u, flags [%s], proto %s (%u)",
                          EXTRACT_16BITS(&ipds->ip->ip_id),
                          (ipds->off & 0x1fff) * 8,
                          bittok2str(ip_frag_values, "none", ipds->off&0xe000),
                          tok2str(ipproto_values,"unknown",ipds->ip->ip_p),
-                         ipds->ip->ip_p);
+                         ipds->ip->ip_p));
 
-            (void)printf(", length %u", EXTRACT_16BITS(&ipds->ip->ip_len));
+            ND_PRINT((ndo, ", length %u", EXTRACT_16BITS(&ipds->ip->ip_len)));
 
             if ((hlen - sizeof(struct ip)) > 0) {
-                printf(", options (");
-                ip_optprint((u_char *)(ipds->ip + 1), hlen - sizeof(struct ip));
-                printf(")");
+                ND_PRINT((ndo, ", options ("));
+                ip_optprint(ndo, (u_char *)(ipds->ip + 1), hlen - sizeof(struct ip));
+                ND_PRINT((ndo, ")"));
             }
 
-	    if (!Kflag && (u_char *)ipds->ip + hlen <= ndo->ndo_snapend) {
-	        vec[0].ptr = (const u_int8_t *)(void *)ipds->ip;
+	    if (!ndo->ndo_Kflag && (u_char *)ipds->ip + hlen <= ndo->ndo_snapend) {
+	        vec[0].ptr = (const uint8_t *)(void *)ipds->ip;
 	        vec[0].len = hlen;
 	        sum = in_cksum(vec, 1);
 		if (sum != 0) {
 		    ip_sum = EXTRACT_16BITS(&ipds->ip->ip_sum);
-		    (void)printf(", bad cksum %x (->%x)!", ip_sum,
-			     in_cksum_shouldbe(ip_sum, sum));
+		    ND_PRINT((ndo, ", bad cksum %x (->%x)!", ip_sum,
+			     in_cksum_shouldbe(ip_sum, sum)));
 		}
 	    }
 
-            printf(")\n    ");
+		ND_PRINT((ndo, ")\n    "));
 	}
 
 	/*
@@ -644,53 +646,52 @@ ip_print(netdissect_options *ndo,
 
 		if (ipds->nh != IPPROTO_TCP && ipds->nh != IPPROTO_UDP &&
 		    ipds->nh != IPPROTO_SCTP && ipds->nh != IPPROTO_DCCP) {
-			(void)printf("%s > %s: ",
-				     ipaddr_string(&ipds->ip->ip_src),
-				     ipaddr_string(&ipds->ip->ip_dst));
+			ND_PRINT((ndo, "%s > %s: ",
+				     ipaddr_string(ndo, &ipds->ip->ip_src),
+				     ipaddr_string(ndo, &ipds->ip->ip_dst)));
 		}
 		ip_print_demux(ndo, ipds);
 	} else {
 	    /* Ultra quiet now means that all this stuff should be suppressed */
-	    if (qflag > 1) return;
+	    if (ndo->ndo_qflag > 1) return;
 
 	    /*
 	     * if this isn't the first frag, we're missing the
 	     * next level protocol header.  print the ip addr
 	     * and the protocol.
 	     */
-	    if (ipds->off & 0x1fff) {
-	        (void)printf("%s > %s:", ipaddr_string(&ipds->ip->ip_src),
-			     ipaddr_string(&ipds->ip->ip_dst));
-		if (!ndo->ndo_nflag && (proto = getprotobynumber(ipds->ip->ip_p)) != NULL)
-		    (void)printf(" %s", proto->p_name);
-		else
-		    (void)printf(" ip-proto-%d", ipds->ip->ip_p);
-	    }
+		if (ipds->off & 0x1fff) {
+			ND_PRINT((ndo, "%s > %s:", ipaddr_string(ndo, &ipds->ip->ip_src),
+			          ipaddr_string(ndo, &ipds->ip->ip_dst)));
+			if (!ndo->ndo_nflag && (proto = getprotobynumber(ipds->ip->ip_p)) != NULL)
+				ND_PRINT((ndo, " %s", proto->p_name));
+			else
+				ND_PRINT((ndo, " ip-proto-%d", ipds->ip->ip_p));
+		}
 	}
 }
 
 void
-ipN_print(register const u_char *bp, register u_int length)
+ipN_print(netdissect_options *ndo, register const u_char *bp, register u_int length)
 {
-	struct ip *ip, hdr;
+	struct ip hdr;
 
-	ip = (struct ip *)bp;
 	if (length < 4) {
-		(void)printf("truncated-ip %d", length);
+		ND_PRINT((ndo, "truncated-ip %d", length));
 		return;
 	}
-	memcpy (&hdr, (char *)ip, 4);
+	memcpy (&hdr, bp, 4);
 	switch (IP_V(&hdr)) {
 	case 4:
-		ip_print (gndo, bp, length);
+		ip_print (ndo, bp, length);
 		return;
 #ifdef INET6
 	case 6:
-		ip6_print (gndo, bp, length);
+		ip6_print (ndo, bp, length);
 		return;
 #endif
 	default:
-		(void)printf("unknown ip %d", IP_V(&hdr));
+		ND_PRINT((ndo, "unknown ip %d", IP_V(&hdr)));
 		return;
 	}
 }
