@@ -22,20 +22,13 @@
  *	with an awful lot of hacking by Jeffrey Mogul, DECWRL
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-llc.c,v 1.75 2007-04-13 09:43:11 hannes Exp $";
-#endif
-
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <tcpdump-stdinc.h>
 
-#include <stdio.h>
-#include <string.h>
-
 #include "interface.h"
 #include "addrtoname.h"
 #include "extract.h"			/* must come after interface.h */
@@ -44,7 +37,7 @@ static const char rcsid[] _U_ =
 #include "ethertype.h"
 #include "oui.h"
 
-static struct tok llc_values[] = {
+static const struct tok llc_values[] = {
         { LLCSAP_NULL,     "Null" },
         { LLCSAP_GLOBAL,   "Global" },
         { LLCSAP_8021B_I,  "802.1B I" },
@@ -63,7 +56,7 @@ static struct tok llc_values[] = {
         { 0,               NULL },
 };
 
-static struct tok llc_cmd_values[] = {
+static const struct tok llc_cmd_values[] = {
 	{ LLC_UI,	"ui" },
 	{ LLC_TEST,	"test" },
 	{ LLC_XID,	"xid" },
@@ -75,7 +68,7 @@ static struct tok llc_cmd_values[] = {
 	{ 0,		NULL }
 };
 
-static const struct tok llc_flag_values[] = { 
+static const struct tok llc_flag_values[] = {
         { 0, "Command" },
         { LLC_GSAP, "Response" },
         { LLC_U_POLL, "Poll" },
@@ -86,14 +79,14 @@ static const struct tok llc_flag_values[] = {
 };
 
 
-static const struct tok llc_ig_flag_values[] = { 
+static const struct tok llc_ig_flag_values[] = {
         { 0, "Individual" },
         { LLC_IG, "Group" },
 	{ 0, NULL }
 };
 
 
-static const struct tok llc_supervisory_values[] = { 
+static const struct tok llc_supervisory_values[] = {
         { 0, "Receiver Ready" },
         { 1, "Receiver not Ready" },
         { 2, "Reject" },
@@ -101,16 +94,17 @@ static const struct tok llc_supervisory_values[] = {
 };
 
 
-static const struct tok cisco_values[] = { 
+static const struct tok cisco_values[] = {
 	{ PID_CISCO_CDP, "CDP" },
 	{ PID_CISCO_VTP, "VTP" },
 	{ PID_CISCO_DTP, "DTP" },
 	{ PID_CISCO_UDLD, "UDLD" },
 	{ PID_CISCO_PVST, "PVST" },
+	{ PID_CISCO_VLANBRIDGE, "VLAN Bridge" },
 	{ 0,             NULL }
 };
 
-static const struct tok bridged_values[] = { 
+static const struct tok bridged_values[] = {
 	{ PID_RFC2684_ETH_FCS,     "Ethernet + FCS" },
 	{ PID_RFC2684_ETH_NOFCS,   "Ethernet w/o FCS" },
 	{ PID_RFC2684_802_4_FCS,   "802.4 + FCS" },
@@ -125,12 +119,12 @@ static const struct tok bridged_values[] = {
 	{ 0,                       NULL },
 };
 
-static const struct tok null_values[] = { 
+static const struct tok null_values[] = {
 	{ 0,             NULL }
 };
 
 struct oui_tok {
-	u_int32_t	oui;
+	uint32_t	oui;
 	const struct tok *tok;
 };
 
@@ -147,19 +141,19 @@ static const struct oui_tok oui_to_tok[] = {
  * Returns non-zero IFF it succeeds in printing the header
  */
 int
-llc_print(const u_char *p, u_int length, u_int caplen,
+llc_print(netdissect_options *ndo, const u_char *p, u_int length, u_int caplen,
 	  const u_char *esrc, const u_char *edst, u_short *extracted_ethertype)
 {
-	u_int8_t dsap_field, dsap, ssap_field, ssap;
-	u_int16_t control;
+	uint8_t dsap_field, dsap, ssap_field, ssap;
+	uint16_t control;
 	int is_u;
 	register int ret;
 
 	*extracted_ethertype = 0;
 
 	if (caplen < 3) {
-		(void)printf("[|llc]");
-		default_print((u_char *)p, caplen);
+		ND_PRINT((ndo, "[|llc]"));
+		ND_DEFAULTPRINT((u_char *)p, caplen);
 		return(0);
 	}
 
@@ -184,8 +178,8 @@ llc_print(const u_char *p, u_int length, u_int caplen,
 		 * 2 bytes...
 		 */
 		if (caplen < 4) {
-			(void)printf("[|llc]");
-			default_print((u_char *)p, caplen);
+			ND_PRINT((ndo, "[|llc]"));
+			ND_DEFAULTPRINT((u_char *)p, caplen);
 			return(0);
 		}
 
@@ -212,41 +206,41 @@ llc_print(const u_char *p, u_int length, u_int caplen,
 		 * least one capture file.)
 		 */
 
-            if (eflag)
-		printf("IPX 802.3: ");
+            if (ndo->ndo_eflag)
+		ND_PRINT((ndo, "IPX 802.3: "));
 
-            ipx_print(p, length);
+            ipx_print(ndo, p, length);
             return (1);
 	}
 
 	dsap = dsap_field & ~LLC_IG;
 	ssap = ssap_field & ~LLC_GSAP;
 
-	if (eflag) {
-                printf("LLC, dsap %s (0x%02x) %s, ssap %s (0x%02x) %s",
+	if (ndo->ndo_eflag) {
+                ND_PRINT((ndo, "LLC, dsap %s (0x%02x) %s, ssap %s (0x%02x) %s",
                        tok2str(llc_values, "Unknown", dsap),
                        dsap,
                        tok2str(llc_ig_flag_values, "Unknown", dsap_field & LLC_IG),
                        tok2str(llc_values, "Unknown", ssap),
                        ssap,
-                       tok2str(llc_flag_values, "Unknown", ssap_field & LLC_GSAP));
+                       tok2str(llc_flag_values, "Unknown", ssap_field & LLC_GSAP)));
 
 		if (is_u) {
-			printf(", ctrl 0x%02x: ", control);
+			ND_PRINT((ndo, ", ctrl 0x%02x: ", control));
 		} else {
-			printf(", ctrl 0x%04x: ", control);
+			ND_PRINT((ndo, ", ctrl 0x%04x: ", control));
 		}
 	}
 
 	if (ssap == LLCSAP_8021D && dsap == LLCSAP_8021D &&
 	    control == LLC_UI) {
-		stp_print(p+3, length-3);
+		stp_print(ndo, p+3, length-3);
 		return (1);
 	}
 
 	if (ssap == LLCSAP_IP && dsap == LLCSAP_IP &&
 	    control == LLC_UI) {
-		ip_print(gndo, p+4, length-4);
+		ip_print(ndo, p+4, length-4);
 		return (1);
 	}
 
@@ -259,10 +253,10 @@ llc_print(const u_char *p, u_int length, u_int caplen,
 		 *
 		 * Skip DSAP, LSAP, and control field.
 		 */
-                if (eflag)
-                        printf("IPX 802.2: ");
+                if (ndo->ndo_eflag)
+                        ND_PRINT((ndo, "IPX 802.2: "));
 
-		ipx_print(p+3, length-3);
+		ipx_print(ndo, p+3, length-3);
 		return (1);
 	}
 
@@ -286,19 +280,17 @@ llc_print(const u_char *p, u_int length, u_int caplen,
 		if (is_u) {
 			p += 3;
 			length -= 3;
-			caplen -= 3;
 		} else {
 			p += 4;
 			length -= 4;
-			caplen -= 4;
 		}
-		netbeui_print(control, p, length);
+		netbeui_print(ndo, control, p, length);
 		return (1);
 	}
 #endif
 	if (ssap == LLCSAP_ISONS && dsap == LLCSAP_ISONS
 	    && control == LLC_UI) {
-		isoclns_print(p + 3, length - 3, caplen - 3);
+		isoclns_print(ndo, p + 3, length - 3, caplen - 3);
 		return (1);
 	}
 
@@ -309,85 +301,77 @@ llc_print(const u_char *p, u_int length, u_int caplen,
 		 * Does anybody ever bridge one form of LAN traffic
 		 * over a networking type that uses 802.2 LLC?
 		 */
-		ret = snap_print(p+3, length-3, caplen-3, 2);
+		ret = snap_print(ndo, p+3, length-3, caplen-3, 2);
 		if (ret)
 			return (ret);
 	}
 
-	if (!eflag) {
+	if (!ndo->ndo_eflag) {
 		if (ssap == dsap) {
 			if (esrc == NULL || edst == NULL)
-				(void)printf("%s ", tok2str(llc_values, "Unknown DSAP 0x%02x", dsap));
+				ND_PRINT((ndo, "%s ", tok2str(llc_values, "Unknown DSAP 0x%02x", dsap)));
 			else
-				(void)printf("%s > %s %s ",
-						etheraddr_string(esrc),
-						etheraddr_string(edst),
-						tok2str(llc_values, "Unknown DSAP 0x%02x", dsap));
+				ND_PRINT((ndo, "%s > %s %s ",
+						etheraddr_string(ndo, esrc),
+						etheraddr_string(ndo, edst),
+						tok2str(llc_values, "Unknown DSAP 0x%02x", dsap)));
 		} else {
 			if (esrc == NULL || edst == NULL)
-				(void)printf("%s > %s ",
+				ND_PRINT((ndo, "%s > %s ",
                                         tok2str(llc_values, "Unknown SSAP 0x%02x", ssap),
-					tok2str(llc_values, "Unknown DSAP 0x%02x", dsap));
+					tok2str(llc_values, "Unknown DSAP 0x%02x", dsap)));
 			else
-				(void)printf("%s %s > %s %s ",
-					etheraddr_string(esrc),
+				ND_PRINT((ndo, "%s %s > %s %s ",
+					etheraddr_string(ndo, esrc),
                                         tok2str(llc_values, "Unknown SSAP 0x%02x", ssap),
-					etheraddr_string(edst),
-					tok2str(llc_values, "Unknown DSAP 0x%02x", dsap));
+					etheraddr_string(ndo, edst),
+					tok2str(llc_values, "Unknown DSAP 0x%02x", dsap)));
 		}
 	}
 
 	if (is_u) {
-		printf("Unnumbered, %s, Flags [%s], length %u",
+		ND_PRINT((ndo, "Unnumbered, %s, Flags [%s], length %u",
                        tok2str(llc_cmd_values, "%02x", LLC_U_CMD(control)),
                        tok2str(llc_flag_values,"?",(ssap_field & LLC_GSAP) | (control & LLC_U_POLL)),
-                       length);
+                       length));
 
 		p += 3;
-		length -= 3;
-		caplen -= 3;
 
 		if ((control & ~LLC_U_POLL) == LLC_XID) {
 			if (*p == LLC_XID_FI) {
-				printf(": %02x %02x", p[1], p[2]);
-				p += 3;
-				length -= 3;
-				caplen -= 3;
+				ND_PRINT((ndo, ": %02x %02x", p[1], p[2]));
 			}
 		}
 	} else {
 		if ((control & LLC_S_FMT) == LLC_S_FMT) {
-			(void)printf("Supervisory, %s, rcv seq %u, Flags [%s], length %u",
+			ND_PRINT((ndo, "Supervisory, %s, rcv seq %u, Flags [%s], length %u",
 				tok2str(llc_supervisory_values,"?",LLC_S_CMD(control)),
 				LLC_IS_NR(control),
 				tok2str(llc_flag_values,"?",(ssap_field & LLC_GSAP) | (control & LLC_IS_POLL)),
-                                length);
+                                length));
 		} else {
-			(void)printf("Information, send seq %u, rcv seq %u, Flags [%s], length %u",
+			ND_PRINT((ndo, "Information, send seq %u, rcv seq %u, Flags [%s], length %u",
 				LLC_I_NS(control),
 				LLC_IS_NR(control),
 				tok2str(llc_flag_values,"?",(ssap_field & LLC_GSAP) | (control & LLC_IS_POLL)),
-                                length);
+                                length));
 		}
-		p += 4;
-		length -= 4;
-		caplen -= 4;
 	}
 	return(1);
 }
 
 int
-snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
+snap_print(netdissect_options *ndo, const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 {
-	u_int32_t orgcode;
+	uint32_t orgcode;
 	register u_short et;
 	register int ret;
 
-	TCHECK2(*p, 5);
+	ND_TCHECK2(*p, 5);
 	orgcode = EXTRACT_24BITS(p);
 	et = EXTRACT_16BITS(p + 3);
 
-	if (eflag) {
+	if (ndo->ndo_eflag) {
 		const struct tok *tok = null_values;
 		const struct oui_tok *otp;
 
@@ -397,12 +381,12 @@ snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 				break;
 			}
 		}
-		(void)printf("oui %s (0x%06x), %s %s (0x%04x): ",
+		ND_PRINT((ndo, "oui %s (0x%06x), %s %s (0x%04x): ",
 		     tok2str(oui_values, "Unknown", orgcode),
 		     orgcode,
 		     (orgcode == 0x000000 ? "ethertype" : "pid"),
 		     tok2str(tok, "Unknown", et),
-		     et);
+		     et));
 	}
 	p += 5;
 	length -= 5;
@@ -417,7 +401,7 @@ snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 		 * Cisco hardware; the protocol ID is
 		 * an Ethernet protocol type.
 		 */
-		ret = ethertype_print(gndo, et, p, length, caplen);
+		ret = ethertype_print(ndo, et, p, length, caplen);
 		if (ret)
 			return (ret);
 		break;
@@ -432,7 +416,7 @@ snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 			 * but used 0x000000 and an Ethernet
 			 * packet type for AARP packets.
 			 */
-			ret = ethertype_print(gndo, et, p, length, caplen);
+			ret = ethertype_print(ndo, et, p, length, caplen);
 			if (ret)
 				return (ret);
 		}
@@ -441,23 +425,25 @@ snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 	case OUI_CISCO:
                 switch (et) {
                 case PID_CISCO_CDP:
-                        cdp_print(p, length, caplen);
+                        cdp_print(ndo, p, length, caplen);
                         return (1);
                 case PID_CISCO_DTP:
-                        dtp_print(p, length); 
+                        dtp_print(ndo, p, length);
                         return (1);
                 case PID_CISCO_UDLD:
-                        udld_print(p, length);
+                        udld_print(ndo, p, length);
                         return (1);
                 case PID_CISCO_VTP:
-                        vtp_print(p, length);
+                        vtp_print(ndo, p, length);
                         return (1);
                 case PID_CISCO_PVST:
-                        stp_print(p, length);
+                case PID_CISCO_VLANBRIDGE:
+                        stp_print(ndo, p, length);
                         return (1);
                 default:
                         break;
                 }
+		break;
 
 	case OUI_RFC2684:
 		switch (et) {
@@ -471,7 +457,7 @@ snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 			/*
 			 * Skip the padding.
 			 */
-			TCHECK2(*p, bridge_pad);
+			ND_TCHECK2(*p, bridge_pad);
 			caplen -= bridge_pad;
 			length -= bridge_pad;
 			p += bridge_pad;
@@ -479,7 +465,7 @@ snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 			/*
 			 * What remains is an Ethernet packet.
 			 */
-			ether_print(gndo, p, length, caplen, NULL, NULL);
+			ether_print(ndo, p, length, caplen, NULL, NULL);
 			return (1);
 
 		case PID_RFC2684_802_5_FCS:
@@ -492,7 +478,7 @@ snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 			 * Skip the padding, but not the Access
 			 * Control field.
 			 */
-			TCHECK2(*p, bridge_pad);
+			ND_TCHECK2(*p, bridge_pad);
 			caplen -= bridge_pad;
 			length -= bridge_pad;
 			p += bridge_pad;
@@ -501,7 +487,7 @@ snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 			 * What remains is an 802.5 Token Ring
 			 * packet.
 			 */
-			token_print(p, length, caplen);
+			token_print(ndo, p, length, caplen);
 			return (1);
 
 		case PID_RFC2684_FDDI_FCS:
@@ -513,7 +499,7 @@ snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 			/*
 			 * Skip the padding.
 			 */
-			TCHECK2(*p, bridge_pad + 1);
+			ND_TCHECK2(*p, bridge_pad + 1);
 			caplen -= bridge_pad + 1;
 			length -= bridge_pad + 1;
 			p += bridge_pad + 1;
@@ -521,18 +507,18 @@ snap_print(const u_char *p, u_int length, u_int caplen, u_int bridge_pad)
 			/*
 			 * What remains is an FDDI packet.
 			 */
-			fddi_print(p, length, caplen);
+			fddi_print(ndo, p, length, caplen);
 			return (1);
 
 		case PID_RFC2684_BPDU:
-			stp_print(p, length);
+			stp_print(ndo, p, length);
 			return (1);
 		}
 	}
 	return (0);
 
 trunc:
-	(void)printf("[|snap]");
+	ND_PRINT((ndo, "[|snap]"));
 	return (1);
 }
 
