@@ -19,44 +19,51 @@
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-sl.c,v 1.65 2005-04-06 21:32:42 mcr Exp $ (LBL)";
-#endif
-
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <tcpdump-stdinc.h>
 
-#include <pcap.h>
-#include <stdio.h>
-
 #include "interface.h"
-#include "addrtoname.h"
 #include "extract.h"			/* must come after interface.h */
 
 #include "ip.h"
 #include "tcp.h"
-#include "slip.h"
 #include "slcompress.h"
 
+/*
+ * definitions of the pseudo- link-level header attached to slip
+ * packets grabbed by the packet filter (bpf) traffic monitor.
+ */
+#define SLIP_HDRLEN 16
+
+#define SLX_DIR 0
+#define SLX_CHDR 1
+#define CHDR_LEN 15
+
+#define SLIPDIR_IN 0
+#define SLIPDIR_OUT 1
+
+static const char tstr[] = "[|slip]";
+
 static u_int lastlen[2][256];
 static u_int lastconn = 255;
 
-static void sliplink_print(const u_char *, const struct ip *, u_int);
-static void compressed_sl_print(const u_char *, const struct ip *, u_int, int);
+static void sliplink_print(netdissect_options *, const u_char *, const struct ip *, u_int);
+static void compressed_sl_print(netdissect_options *, const u_char *, const struct ip *, u_int, int);
 
 u_int
-sl_if_print(const struct pcap_pkthdr *h, const u_char *p)
+sl_if_print(netdissect_options *ndo,
+            const struct pcap_pkthdr *h, const u_char *p)
 {
 	register u_int caplen = h->caplen;
 	register u_int length = h->len;
 	register const struct ip *ip;
 
 	if (caplen < SLIP_HDRLEN) {
-		printf("[|slip]");
+		ND_PRINT((ndo, "%s", tstr));
 		return (caplen);
 	}
 
@@ -64,34 +71,35 @@ sl_if_print(const struct pcap_pkthdr *h, const u_char *p)
 
 	ip = (struct ip *)(p + SLIP_HDRLEN);
 
-	if (eflag)
-		sliplink_print(p, ip, length);
+	if (ndo->ndo_eflag)
+		sliplink_print(ndo, p, ip, length);
 
 	switch (IP_V(ip)) {
 	case 4:
-	        ip_print(gndo, (u_char *)ip, length);
+	        ip_print(ndo, (u_char *)ip, length);
 		break;
 #ifdef INET6
 	case 6:
-		ip6_print(gndo, (u_char *)ip, length);
+		ip6_print(ndo, (u_char *)ip, length);
 		break;
 #endif
 	default:
-		printf ("ip v%d", IP_V(ip));
+		ND_PRINT((ndo, "ip v%d", IP_V(ip)));
 	}
 
 	return (SLIP_HDRLEN);
 }
 
 u_int
-sl_bsdos_if_print(const struct pcap_pkthdr *h, const u_char *p)
+sl_bsdos_if_print(netdissect_options *ndo,
+                  const struct pcap_pkthdr *h, const u_char *p)
 {
 	register u_int caplen = h->caplen;
 	register u_int length = h->len;
 	register const struct ip *ip;
 
 	if (caplen < SLIP_HDRLEN) {
-		printf("[|slip]");
+		ND_PRINT((ndo, "%s", tstr));
 		return (caplen);
 	}
 
@@ -100,39 +108,39 @@ sl_bsdos_if_print(const struct pcap_pkthdr *h, const u_char *p)
 	ip = (struct ip *)(p + SLIP_HDRLEN);
 
 #ifdef notdef
-	if (eflag)
-		sliplink_print(p, ip, length);
+	if (ndo->ndo_eflag)
+		sliplink_print(ndo, p, ip, length);
 #endif
 
-	ip_print(gndo, (u_char *)ip, length);
+	ip_print(ndo, (u_char *)ip, length);
 
 	return (SLIP_HDRLEN);
 }
 
 static void
-sliplink_print(register const u_char *p, register const struct ip *ip,
-	       register u_int length)
+sliplink_print(netdissect_options *ndo,
+               register const u_char *p, register const struct ip *ip,
+               register u_int length)
 {
 	int dir;
 	u_int hlen;
 
 	dir = p[SLX_DIR];
-	putchar(dir == SLIPDIR_IN ? 'I' : 'O');
-	putchar(' ');
+	ND_PRINT((ndo, dir == SLIPDIR_IN ? "I " : "O "));
 
-	if (nflag) {
+	if (ndo->ndo_nflag) {
 		/* XXX just dump the header */
 		register int i;
 
 		for (i = SLX_CHDR; i < SLX_CHDR + CHDR_LEN - 1; ++i)
-			printf("%02x.", p[i]);
-		printf("%02x: ", p[SLX_CHDR + CHDR_LEN - 1]);
+			ND_PRINT((ndo, "%02x.", p[i]));
+		ND_PRINT((ndo, "%02x: ", p[SLX_CHDR + CHDR_LEN - 1]));
 		return;
 	}
 	switch (p[SLX_CHDR] & 0xf0) {
 
 	case TYPE_IP:
-		printf("ip %d: ", length + SLIP_HDRLEN);
+		ND_PRINT((ndo, "ip %d: ", length + SLIP_HDRLEN));
 		break;
 
 	case TYPE_UNCOMPRESSED_TCP:
@@ -145,21 +153,22 @@ sliplink_print(register const u_char *p, register const struct ip *ip,
 		hlen = IP_HL(ip);
 		hlen += TH_OFF((struct tcphdr *)&((int *)ip)[hlen]);
 		lastlen[dir][lastconn] = length - (hlen << 2);
-		printf("utcp %d: ", lastconn);
+		ND_PRINT((ndo, "utcp %d: ", lastconn));
 		break;
 
 	default:
 		if (p[SLX_CHDR] & TYPE_COMPRESSED_TCP) {
-			compressed_sl_print(&p[SLX_CHDR], ip,
+			compressed_sl_print(ndo, &p[SLX_CHDR], ip,
 			    length, dir);
-			printf(": ");
+			ND_PRINT((ndo, ": "));
 		} else
-			printf("slip-%d!: ", p[SLX_CHDR]);
+			ND_PRINT((ndo, "slip-%d!: ", p[SLX_CHDR]));
 	}
 }
 
 static const u_char *
-print_sl_change(const char *str, register const u_char *cp)
+print_sl_change(netdissect_options *ndo,
+                const char *str, register const u_char *cp)
 {
 	register u_int i;
 
@@ -167,12 +176,13 @@ print_sl_change(const char *str, register const u_char *cp)
 		i = EXTRACT_16BITS(cp);
 		cp += 2;
 	}
-	printf(" %s%d", str, i);
+	ND_PRINT((ndo, " %s%d", str, i));
 	return (cp);
 }
 
 static const u_char *
-print_sl_winchange(register const u_char *cp)
+print_sl_winchange(netdissect_options *ndo,
+                   register const u_char *cp)
 {
 	register short i;
 
@@ -181,15 +191,16 @@ print_sl_winchange(register const u_char *cp)
 		cp += 2;
 	}
 	if (i >= 0)
-		printf(" W+%d", i);
+		ND_PRINT((ndo, " W+%d", i));
 	else
-		printf(" W%d", i);
+		ND_PRINT((ndo, " W%d", i));
 	return (cp);
 }
 
 static void
-compressed_sl_print(const u_char *chdr, const struct ip *ip,
-		    u_int length, int dir)
+compressed_sl_print(netdissect_options *ndo,
+                    const u_char *chdr, const struct ip *ip,
+                    u_int length, int dir)
 {
 	register const u_char *cp = chdr;
 	register u_int flags, hlen;
@@ -197,35 +208,35 @@ compressed_sl_print(const u_char *chdr, const struct ip *ip,
 	flags = *cp++;
 	if (flags & NEW_C) {
 		lastconn = *cp++;
-		printf("ctcp %d", lastconn);
+		ND_PRINT((ndo, "ctcp %d", lastconn));
 	} else
-		printf("ctcp *");
+		ND_PRINT((ndo, "ctcp *"));
 
 	/* skip tcp checksum */
 	cp += 2;
 
 	switch (flags & SPECIALS_MASK) {
 	case SPECIAL_I:
-		printf(" *SA+%d", lastlen[dir][lastconn]);
+		ND_PRINT((ndo, " *SA+%d", lastlen[dir][lastconn]));
 		break;
 
 	case SPECIAL_D:
-		printf(" *S+%d", lastlen[dir][lastconn]);
+		ND_PRINT((ndo, " *S+%d", lastlen[dir][lastconn]));
 		break;
 
 	default:
 		if (flags & NEW_U)
-			cp = print_sl_change("U=", cp);
+			cp = print_sl_change(ndo, "U=", cp);
 		if (flags & NEW_W)
-			cp = print_sl_winchange(cp);
+			cp = print_sl_winchange(ndo, cp);
 		if (flags & NEW_A)
-			cp = print_sl_change("A+", cp);
+			cp = print_sl_change(ndo, "A+", cp);
 		if (flags & NEW_S)
-			cp = print_sl_change("S+", cp);
+			cp = print_sl_change(ndo, "S+", cp);
 		break;
 	}
 	if (flags & NEW_I)
-		cp = print_sl_change("I+", cp);
+		cp = print_sl_change(ndo, "I+", cp);
 
 	/*
 	 * 'hlen' is the length of the uncompressed TCP/IP header (in words).
@@ -235,5 +246,5 @@ compressed_sl_print(const u_char *chdr, const struct ip *ip,
 	hlen = IP_HL(ip);
 	hlen += TH_OFF((struct tcphdr *)&((int32_t *)ip)[hlen]);
 	lastlen[dir][lastconn] = length - (hlen << 2);
-	printf(" %d (%ld)", lastlen[dir][lastconn], (long)(cp - chdr));
+	ND_PRINT((ndo, " %d (%ld)", lastlen[dir][lastconn], (long)(cp - chdr)));
 }
